
#include <QValidator>
#include <QFile>
#include <QDebug>
#include <QProcess>

#include "sdk-connectivity-tool.h"

#include "editinterface.h"
#include "processrunner.h"


#if 1
#define DEVELOPER_MODE_CONF "/etc/developer-mode.conf"
#else
#undef INTERFACES_SH
#define INTERFACES_SH "../scripts/interfaces.sh"
#define DEVELOPER_MODE_CONF "../scripts/etc/developer-mode.conf"
#endif

class IP4Validator : public QValidator
{
public:
    IP4Validator(QObject *parent = 0) : QValidator(parent) { DIN; DOUT; }
    State validate(QString &input, int &pos) const
    {
        DIN;
        (void)pos;
        if (input.isEmpty()) {
            DOUT;
            return Acceptable;
        }

        QStringList octets = input.split('.');
        int s = octets.size();
        if(s > 4) {
            DOUT;
            return Invalid;
        }

        bool empty = false;
        for (int i=0; i < s; i++) {
            if (octets[i].isEmpty()) {
                empty = true;
                continue;
            }

            bool ok;
            int val = octets[i].toInt(&ok);
            if (!ok || val < 0 || val > 255) {
                DOUT;
                return Invalid;
            }
        }

        if (s < 4 || empty) {
            DOUT;
            return Intermediate;
        }
        DOUT;
        return Acceptable;
    }
};

class IP4MaskValidator : public QValidator
{
public:
    IP4MaskValidator(QObject *parent = 0) : QValidator(parent) { DIN; DOUT; }
    State validate(QString &input, int &pos) const
    {
        DIN;
        (void)pos;
        if (input.isEmpty()) {
            DOUT;
            return Acceptable;
        }

        QStringList octets = input.split('.');
        int s = octets.size();
        if(s > 4) {
            DOUT;
            return Invalid;
        }

        bool empty = false;
        bool zero = false;
        for (int i=0; i < s; i++) {

            if (octets[i].isEmpty()) {
                empty = true;
                continue;
            }
            bool ok;
            int val = octets[i].toInt(&ok);
            if (!ok || (zero && val != 0)) {
                DOUT;
                return Invalid;
            }

            switch (val) {
            case 1:
            case 2:
            case 25:
            case 24:
            case 22:
            case 19:
            case 12:
                if (i < s - 1) {
                    DOUT;
                    return Invalid;
                }
                continue;

            case 255:
                continue;
            case 254:
            case 252:
            case 248:
            case 240:
            case 224:
            case 192:
            case 128:
            case 0:
                zero = true;
                continue;
            default:
                DOUT;
                return Invalid;
            }
        }

        if (s < 4 || empty) {
            DOUT;
            return Intermediate;
        }
        DOUT;
        return Acceptable;
    }
};



EditInterface::EditInterface(QObject *obj, ProcessRunner *runner)
    :m_runner(runner)
{
    DIN;
    connect(m_runner,SIGNAL(finished(int)),this,SIGNAL(finished(int)));
    DOUT;
}

EditInterface::~EditInterface()
{
    DIN;
    DOUT;
}


void
EditInterface::read_config(NetI &neti)
{
    DIN;
    neti.address = "192.168.2.15";
    neti.netmask = "255.255.255.0";
    neti.peerip = "192.168.2.14";
    neti.dhcpValue= true;

    QFile file(DEVELOPER_MODE_CONF);

    if (file.open(QIODevice::ReadOnly))
    {
        while (!file.atEnd()) {

            QList<QByteArray> list = file.readLine().split('\'');

            if (list.size() != 3)
                continue;

            if (list[0] == "cnf_usb0_address=") {
                neti.address = list[1];
            }else if (list[0] == "cnf_usb0_netmask=")  {
                neti.netmask = list[1];
            }else if (list[0] == "cnf_usb0_gateway="){
                neti.gateway = list[1];
            }else if (list[0] == "cnf_usb0_peerip="){
                neti.peerip = list[1];
            } else if (list[0] == "cnf_usb0_mode=") {
                if (list[1] == "static")
                {
                    neti.dhcpValue = false;
                }
                else
                {
                    neti.dhcpValue = true;
                }
            }
        }
    }
    DOUT;
}


void
EditInterface::updateUsbConfiguration(QString usbIpVal, QString usbNetmaskVal, QString peerIpVal, bool dhcpSelect, int configtypeValue)//RAJ_11APR
{
    DIN;
    /*Removed else block as this function will be called only if usb is selected
    and instead of default values we are assigning values provided by user in edit settings*/
    usbIp = usbIpVal;
    usbNetmask = usbNetmaskVal;
    peerIp = peerIpVal;
    dhcpSelectVal = dhcpSelect;
    DOUT;
}

void
EditInterface::configureQml()
{
    DIN;
    QString name(INTERFACES_SH);
    QStringList args(QString("usb0"));

    if( (this->usbIp.contains("-") ) || (this->usbIp.isEmpty()) || this->usbIp.isNull() )
    {
        this->usbIp = "192.168.2.15";
    }
    if( (this->usbNetmask.contains("-") ) || (this->usbNetmask.isEmpty()) || this->usbNetmask.isNull() )
    {
        this->usbNetmask = "255.255.255.0";
    }

    if( (this->peerIp.contains("-") ) || (this->peerIp.isEmpty()) || this->peerIp.isNull() )
    {
        this->peerIp = "192.168.2.14";
    }

    if(dhcpSelectVal == true)
    {
        args.append("dhcpd");
        args.append(usbIp);
        args.append(usbNetmask);
        args.append(peerIp);

    }else
    {
        args.append("static");
        args.append(usbIp);
        args.append(usbNetmask);
    }

    m_runner->start(name, args);
    DOUT;
}
